@@ -85,11 +85,14 @@ class ModelArgs:
     skip_special_tokens: bool = True
     tensorboard_dir: str = None
     thread_count: int = None
+    tokenizer_type: str = None
+    tokenizer_name: str = None
     train_batch_size: int = 8
     train_custom_parameters_only: bool = False
     use_cached_eval_features: bool = False
     use_early_stopping: bool = False
     use_multiprocessing: bool = True
+    use_multiprocessing_for_evaluation: bool = False
     wandb_kwargs: dict = field(default_factory=dict)
     wandb_project: str = None
     warmup_ratio: float = 0.06
@@ -110,7 +113,10 @@ class ModelArgs:
     def save(self, output_dir):
         os.makedirs(output_dir, exist_ok=True)
         with open(os.path.join(output_dir, "model_args.json"), "w") as f:
-            json.dump(self.get_args_for_saving(), f)
+            args_dict = self.get_args_for_saving()
+            if args_dict["tokenizer_type"] is not None and not isinstance(args_dict["tokenizer_type"], str):
+                args_dict["tokenizer_type"] = type(args_dict["tokenizer_type"]).__name__
+            json.dump(args_dict, f)
 
     def load(self, input_dir):
         if input_dir:
@@ -141,6 +147,7 @@ class ClassificationArgs(ModelArgs):
     onnx: bool = False
     regression: bool = False
     sliding_window: bool = False
+    special_tokens_list: list = field(default_factory=list)
     stride: float = 0.8
     tie_value: int = 1
 
@@ -159,6 +166,7 @@ class MultiLabelClassificationArgs(ModelArgs):
     labels_list: list = field(default_factory=list)
     labels_map: dict = field(default_factory=dict)
     lazy_loading: bool = False
+    special_tokens_list: list = field(default_factory=list)
 
 
 @dataclass
@@ -173,6 +181,7 @@ class NERArgs(ModelArgs):
     lazy_loading: bool = False
     lazy_loading_start_line: int = 0
     onnx: bool = False
+    special_tokens_list: list = field(default_factory=list)
 
 
 @dataclass
@@ -190,6 +199,7 @@ class QuestionAnsweringArgs(ModelArgs):
     max_query_length: int = 64
     n_best_size: int = 20
     null_score_diff_threshold: float = 0.0
+    special_tokens_list: list = field(default_factory=list)
 
 
 @dataclass
@@ -216,6 +226,7 @@ class T5Args(ModelArgs):
     adafactor_warmup_init: bool = False
     learning_rate: float = 1e-3
     optimizer: str = "Adafactor"
+    special_tokens_list: list = field(default_factory=list)
     top_k: float = None
     top_p: float = None
     use_multiprocessed_decoding: bool = True
@@ -247,6 +258,7 @@ class LanguageModelingArgs(ModelArgs):
     vocab_size: int = None
     clean_text: bool = True
     handle_chinese_chars: bool = True
+    special_tokens_list: list = field(default_factory=list)
     strip_accents: bool = True
     local_rank: int = -1
 
@@ -345,6 +357,7 @@ class LanguageGenerationArgs(ModelArgs):
     xlm_language: str = ""
     config_name: str = None
     tokenizer_name: str = None
+    special_tokens_list: list = field(default_factory=list)
 
 
 @dataclass
@@ -381,3 +394,4 @@ class MultiModalClassificationArgs(ModelArgs):
     images_label: str = "images"
     image_type_extension: str = ""
     data_type_extension: str = ""
+    special_tokens_list: list = field(default_factory=list)
