@@ -30,7 +30,9 @@ from transformers.optimization import (
 )
 from transformers.optimization import AdamW, Adafactor
 from transformers import (
-    WEIGHTS_NAME,
+    AlbertConfig,
+    AlbertForTokenClassification,
+    AlbertTokenizer,
     AutoConfig,
     AutoModelForTokenClassification,
     AutoTokenizer,
@@ -47,21 +49,28 @@ from transformers import (
     ElectraConfig,
     ElectraForTokenClassification,
     ElectraTokenizer,
+    LayoutLMConfig,
+    LayoutLMForTokenClassification,
+    LayoutLMTokenizer,
     LongformerConfig,
     LongformerForTokenClassification,
     LongformerTokenizer,
+    MPNetConfig,
+    MPNetForTokenClassification,
+    MPNetTokenizer,
     MobileBertConfig,
     MobileBertForTokenClassification,
     MobileBertTokenizer,
     RobertaConfig,
     RobertaForTokenClassification,
     RobertaTokenizer,
+    SqueezeBertConfig,
+    SqueezeBertForTokenClassification,
+    SqueezeBertTokenizer,
+    WEIGHTS_NAME,
     XLMRobertaConfig,
     XLMRobertaForTokenClassification,
     XLMRobertaTokenizer,
-    LayoutLMConfig,
-    LayoutLMForTokenClassification,
-    LayoutLMTokenizer,
 )
 from wandb import config
 from transformers.convert_graph_to_onnx import convert, quantize
@@ -88,6 +97,9 @@ except ImportError:
 logger = logging.getLogger(__name__)
 
 
+MODELS_WITH_EXTRA_SEP_TOKEN = ["roberta", "camembert", "xlmroberta", "longformer", "mpnet"]
+
+
 class NERModel:
     def __init__(
         self,
@@ -114,16 +126,20 @@ class NERModel:
         """  # noqa: ignore flake8"
 
         MODEL_CLASSES = {
-            "auto": (AutoConfig, AutoTokenizer, AutoModelForTokenClassification),
+            "albert": (AlbertConfig, AlbertForTokenClassification, AlbertTokenizer),
+            "auto": (AutoConfig, AutoModelForTokenClassification, AutoTokenizer),
             "bert": (BertConfig, BertForTokenClassification, BertTokenizer),
             "bertweet": (RobertaConfig, RobertaForTokenClassification, BertweetTokenizer),
             "camembert": (CamembertConfig, CamembertForTokenClassification, CamembertTokenizer),
             "distilbert": (DistilBertConfig, DistilBertForTokenClassification, DistilBertTokenizer),
             "electra": (ElectraConfig, ElectraForTokenClassification, ElectraTokenizer),
-            "longformer": (LongformerConfig, LongformerForTokenClassification, LongformerTokenizer),
-            "roberta": (RobertaConfig, RobertaForTokenClassification, RobertaTokenizer),
-            "xlmroberta": (XLMRobertaConfig, XLMRobertaForTokenClassification, XLMRobertaTokenizer),
             "layoutlm": (LayoutLMConfig, LayoutLMForTokenClassification, LayoutLMTokenizer),
+            "longformer": (LongformerConfig, LongformerForTokenClassification, LongformerTokenizer),
+            "mobilebert": (MobileBertConfig, MobileBertForTokenClassification, MobileBertTokenizer),
+            "mpnet": (MPNetConfig, MPNetForTokenClassification, MPNetTokenizer),
+            "roberta": (RobertaConfig, RobertaForTokenClassification, RobertaTokenizer),
+            "squeezebert": (SqueezeBertConfig, SqueezeBertForTokenClassification, SqueezeBertTokenizer),
+            "xlmroberta": (XLMRobertaConfig, XLMRobertaForTokenClassification, XLMRobertaTokenizer),
         }
 
         self.args = self._load_model_args(model_name)
@@ -244,6 +260,10 @@ class NERModel:
                 model_name, do_lower_case=self.args.do_lower_case, **kwargs
             )
 
+        if self.args.special_tokens_list:
+            self.tokenizer.add_tokens(self.args.special_tokens_list, special_tokens=True)
+            self.model.resize_token_embeddings(len(self.tokenizer))
+
         self.args.model_name = model_name
         self.args.model_type = model_type
 
@@ -690,6 +710,7 @@ class NERModel:
                                             if not self.args.evaluate_during_training
                                             else training_progress_scores,
                                         )
+                        model.train()
 
             epoch_number += 1
             output_dir_current = os.path.join(output_dir, "checkpoint-{}-epoch-{}".format(global_step, epoch_number))
@@ -1217,7 +1238,7 @@ class NERModel:
                     sep_token=tokenizer.sep_token,
                     # RoBERTa uses an extra separator b/w pairs of sentences,
                     # cf. github.com/pytorch/fairseq/commit/1684e166e3da03f5b600dbb7855cb98ddfcd0805
-                    sep_token_extra=bool(args.model_type in ["roberta"]),
+                    sep_token_extra=args.model_type in MODELS_WITH_EXTRA_SEP_TOKEN,
                     # PAD on the left for XLNet
                     pad_on_left=bool(args.model_type in ["xlnet"]),
                     pad_token=tokenizer.convert_tokens_to_ids([tokenizer.pad_token])[0],
@@ -1227,6 +1248,8 @@ class NERModel:
                     silent=args.silent,
                     use_multiprocessing=args.use_multiprocessing,
                     chunksize=args.multiprocessing_chunksize,
+                    mode=mode,
+                    use_multiprocessing_for_evaluation=args.use_multiprocessing_for_evaluation,
                 )
 
                 if not no_cache:
