@@ -11,6 +11,7 @@ from pathlib import Path
 import numpy as np
 import pandas as pd
 import torch
+import transformers
 from tensorboardX import SummaryWriter
 from torch.nn.utils.rnn import pad_sequence
 from torch.utils.data import DataLoader, Dataset, RandomSampler, SequentialSampler
@@ -78,6 +79,9 @@ try:
 except ImportError:
     wandb_available = False
 
+if transformers.__version__ < "4.2.0":
+    MBartForConditionalGeneration._keys_to_ignore_on_save = []
+
 logger = logging.getLogger(__name__)
 
 MODEL_CLASSES = {
@@ -704,6 +708,7 @@ class Seq2SeqModel:
                                             if not self.args.evaluate_during_training
                                             else training_progress_scores,
                                         )
+                        model.train()
 
             epoch_number += 1
             output_dir_current = os.path.join(output_dir, "checkpoint-{}-epoch-{}".format(global_step, epoch_number))
